@@ -19,17 +19,16 @@ from graphene.types.mutation import MutationOptions
 from graphene.types.utils import yank_fields_from_attrs
 from graphene_django.registry import get_global_registry
 from graphene_django.converter import convert_django_field_with_choices
+from graphene.utils.str_converters import to_camel_case
 from graphql.error import GraphQLError
 
 from .exceptions import PermissionDenied
+from .models import GuardedModel
 from .types import (
     MutationErrorType,
     UploadType,
 )
-from .utils import (
-    get_nodes,
-    snake2camelcase,
-)
+from .utils import get_nodes
 
 _registry = get_global_registry()
 
@@ -61,7 +60,7 @@ def _get_validation_errors(validation_error):
                 if field == NON_FIELD_ERRORS:
                     field = None
                 else:
-                    field = snake2camelcase(field)
+                    field = to_camel_case(field)
                 e_list.append(MutationErrorType(field=field, message=e))
     else:
         # convert non-field errors
@@ -326,7 +325,7 @@ class BaseModelMutation(BaseMutation):
         The easiest way when using `guardian` is to inherit it
         from :class:`graphene_django_plus.models.GuardedModel`.
         """
-        if not hasattr(instance, 'has_perm'):
+        if not isinstance(instance, GuardedModel):
             return True
 
         return instance.has_perm(
@@ -360,6 +359,7 @@ class BaseModelMutation(BaseMutation):
         Override this to perform any operation on the instance
         after its `.save()` method is called.
         """
+        pass
 
     @classmethod
     def save(cls, info, instance, cleaned_input=None):
@@ -402,7 +402,7 @@ class BaseModelMutation(BaseMutation):
         cls.after_delete(info, instance)
 
 
-class BaseModelOperationMutation(BaseModelMutation):
+class ModelOperationMutation(BaseModelMutation):
     """Base mutation for operations on models.
 
     Just like a regular :class:`BaseModelMutation`, but this will receive only
@@ -507,8 +507,10 @@ class ModelMutation(BaseModelMutation):
         """
         obj_id = data.get('id')
         if obj_id:
+            checked_permissions = True
             instance = cls.get_instance(info, obj_id)
         else:
+            checked_permissions = False
             instance = cls._meta.model()
 
         cleaned_input = cls.clean_input(info, instance, data)
@@ -526,6 +528,13 @@ class ModelMutation(BaseModelMutation):
             if d is not None:
                 f.save_form_data(instance, d)
 
+        if not checked_permissions:
+            # If we did not check permissions when getting the instance,
+            # check if here. The model might check the permissions based on
+            # some related objects
+            if not cls.check_object_permissions(info.context.user, instance):
+                raise PermissionDenied()
+
         return cls(**{cls._meta.return_field_name: instance})
 
 
@@ -573,7 +582,7 @@ class ModelUpdateMutation(ModelMutation):
         )
 
 
-class ModelDeleteMutation(BaseModelOperationMutation):
+class ModelDeleteMutation(ModelOperationMutation):
     """Delete mutation for models."""
 
     class Meta:
@@ -596,3 +605,7 @@ class ModelDeleteMutation(BaseModelOperationMutation):
         # ID so that the success response contains ID of the deleted object.
         instance.id = db_id
         return cls(**{cls._meta.return_field_name: instance})
+
+
+# Compatibility with older versions
+BaseModelOperationMutation = ModelOperationMutation
